"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectCurrentVersionAsync = exports.monoRepoConfigsAsync = exports.savePxtModulesAsync = exports.saveBuiltFilesAsync = exports.writeFilesAsync = exports.mkHomeCacheAsync = exports.readProjectAsync = exports.readPrjFileAsync = exports.fileExistsAsync = exports.relativePath = exports.findProjectDirAsync = exports.findParentDirWithAsync = void 0;
const path = require("path");
const mkc = require("./mkc");
const host_1 = require("./host");
const semver_1 = require("./semver");
function findParentDirWithAsync(base, filename) {
    return __awaiter(this, void 0, void 0, function* () {
        let s = base;
        while (true) {
            if (yield (0, host_1.host)().existsAsync(path.join(s, filename)))
                return s;
            const s2 = path.resolve(path.join(s, ".."));
            if (s == s2)
                return null;
            s = s2;
        }
    });
}
exports.findParentDirWithAsync = findParentDirWithAsync;
function findProjectDirAsync() {
    return __awaiter(this, void 0, void 0, function* () {
        return findParentDirWithAsync(yield (0, host_1.host)().cwdAsync(), "pxt.json");
    });
}
exports.findProjectDirAsync = findProjectDirAsync;
function resolveFilename(dir, filename) {
    const resolved = path.resolve(dir, filename);
    if (resolved.startsWith(path.resolve(".", dir)))
        return resolved;
    throw new Error(`Invalid file name: ${filename} (in ${dir})`);
}
function relativePath(currdir, target) {
    return path.relative(currdir, target);
}
exports.relativePath = relativePath;
function fileExistsAsync(name) {
    return (0, host_1.host)().existsAsync(name);
}
exports.fileExistsAsync = fileExistsAsync;
function readPrjFileAsync(dir, filename) {
    return (0, host_1.host)().readFileAsync(resolveFilename(dir, filename), "utf8");
}
exports.readPrjFileAsync = readPrjFileAsync;
function readProjectAsync(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        const pxtJson = yield (0, host_1.host)().readFileAsync(path.join(dir, "pxt.json"), "utf8");
        const res = {
            config: JSON.parse(pxtJson),
            mkcConfig: null,
            files: {
                "pxt.json": pxtJson,
            },
        };
        for (const fn of res.config.files.concat(res.config.testFiles || [])) {
            res.files[fn] = yield (0, host_1.host)().readFileAsync(resolveFilename(dir, fn), "utf8");
        }
        return res.files;
    });
}
exports.readProjectAsync = readProjectAsync;
function homePxtDir() {
    return path.join((0, host_1.host)().getEnvironmentVariable("HOME") || (0, host_1.host)().getEnvironmentVariable("UserProfile"), ".pxt");
}
function mkHomeCacheAsync(dir) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!dir)
            dir = homePxtDir();
        yield mkdirpAsync(dir);
        const rootPath = path.join(dir, "mkc-cache");
        yield mkdirpAsync(rootPath);
        function expandKey(key) {
            return key.replace(/[^\.a-z0-9_\-]/g, c => "_" + c.charCodeAt(0) + "_");
        }
        function keyPath(key) {
            return path.join(rootPath, expandKey(key));
        }
        return {
            rootPath,
            expandKey,
            getAsync: key => (0, host_1.host)().readFileAsync(keyPath(key)).then(buf => buf, err => null),
            setAsync: (key, val) => (0, host_1.host)().writeFileAsync(keyPath(key), val),
        };
    });
}
exports.mkHomeCacheAsync = mkHomeCacheAsync;
function mkdirpAsync(dirname, lev = 5) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!(yield (0, host_1.host)().existsAsync(dirname))) {
            if (lev > 0)
                yield mkdirpAsync(path.resolve(dirname, ".."), lev - 1);
            yield (0, host_1.host)().mkdirAsync(dirname);
        }
    });
}
function writeFilesAsync(built, outfiles, log = false) {
    return __awaiter(this, void 0, void 0, function* () {
        yield mkdirpAsync(built);
        for (let fn of Object.keys(outfiles)) {
            if (fn.indexOf("/") >= 0)
                continue;
            if (log)
                mkc.log(`write ${built}/${fn}`);
            if (/\.(uf2|pxt64|elf)$/.test(fn))
                yield (0, host_1.host)().writeFileAsync(path.join(built, fn), outfiles[fn], "base64");
            else
                yield (0, host_1.host)().writeFileAsync(path.join(built, fn), outfiles[fn], "utf8");
        }
    });
}
exports.writeFilesAsync = writeFilesAsync;
function saveBuiltFilesAsync(dir, res, folder = "built") {
    return __awaiter(this, void 0, void 0, function* () {
        yield writeFilesAsync(path.join(dir, folder), res.outfiles || {}, true);
    });
}
exports.saveBuiltFilesAsync = saveBuiltFilesAsync;
function savePxtModulesAsync(dir, files) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const k of Object.keys(files))
            if (k.startsWith("pxt_modules/")) {
                yield mkdirpAsync(path.dirname(k));
                const v = files[k];
                if (typeof v == "string") {
                    mkc.debug(`    write ${k}`);
                    yield (0, host_1.host)().writeFileAsync(k, v);
                }
                else {
                    mkc.debug(`    link ${k}`);
                    try {
                        yield (0, host_1.host)().unlinkAsync(k);
                    }
                    catch (_a) { }
                    yield (0, host_1.host)().symlinkAsync(v.symlink, k, "file");
                }
            }
    });
}
exports.savePxtModulesAsync = savePxtModulesAsync;
function monoRepoConfigsAsync(folder, includingSelf = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield (0, host_1.host)().listFilesAsync(folder, "pxt.json");
        return files.filter(e => e.indexOf("pxt_modules") < 0 &&
            e.indexOf("node_modules") < 0 &&
            (includingSelf ||
                path.resolve(folder, "pxt.json") != path.resolve(e)));
    });
}
exports.monoRepoConfigsAsync = monoRepoConfigsAsync;
function collectCurrentVersionAsync(configs) {
    return __awaiter(this, void 0, void 0, function* () {
        let version = (0, semver_1.tryParse)("0.0.0");
        for (const config of configs) {
            const cfg = JSON.parse(yield (0, host_1.host)().readFileAsync(config, "utf8"));
            const v = (0, semver_1.tryParse)(cfg.version);
            if (v && (0, semver_1.cmp)(version, v) < 0)
                version = v;
        }
        return (0, semver_1.stringify)(version);
    });
}
exports.collectCurrentVersionAsync = collectCurrentVersionAsync;
//# sourceMappingURL=files.js.map